# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-04 15:28:53 +0900 (Sat, 04 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'net/http'

module VFS;  module SchemeHandler;  module HTTP;
  module Common

    def http_fetch(fetch_url, method=:get, &block)
      t_host = parse_host(fetch_url)
      t_port = parse_port(fetch_url)
      t_path = parse_path(fetch_url)
      t_auth = parse_auth(fetch_url) # not supported yet!
      
      Net::HTTP.version_1_1
      begin
	log("Opening HTTP connection.", Log::INFO)
	Net::HTTP::Proxy(parse_host(ENV["parse_proxy"]),
			 parse_port(ENV["parse_proxy"])).start(t_host,
							       t_port) { |http|
	  response = http.send(method, t_path)
	  log("Data was fetched from web server. Go yield.", Log::DEBUG)
	  yield(response)
	}
      rescue Net::ProtoRetriableError => err
	log("Retriable error was detected in HTTP connection. (detail: #{err})",
	    Log::INFO)
	l = err.response['location']
	if parse_host(l)
	  t_host = parse_host(l)
	  t_port = parse_port(l)
	  t_path = parse_path(l)
	  t_auth = parse_auth(l) # not supported yet!
	  retry
	end
      end
    end # http_fetch

    private
    def parse_path(url)
      Regexp.new("^http://[^/]+(.+)$").match(url).to_a[1]
    end
    
    def parse_host(url)
      Regexp.new("^http://(?:[^/@]+?@)?([^/]+)").match(url).to_a[1]
    end
    
    def parse_port(url)
      Regexp.new("^http://[^/]+:([0-9]+)/").match(url).to_a[1] || 80
    end
    
    def parse_auth(url)
      Regexp.new("^http://([^/]+?)@").match(url).to_a[1]
    end

  end # Common
end; end; end
