# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-04 15:28:53 +0900 (Sat, 04 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/file'
require 'vfs/scheme/http/common'
require 'thread'
require 'time.rb'

module VFS;  module SchemeHandler;
  module HTTP;
    class File < VFS::File
      include VFS::SchemeHandler::HTTP::Common
      # FIXME, TODO :utime, :delete, :unlink, :rename, :truncate,
      # :lstat :stat :chown(hmm?) and  write-open!
      # :writable_real?
      # :writable? :umask :exists? :file? :readable_real? :directory?
      # :expand_path :readable? :symlink? :readlink

      return_false :chardev?, :pipe?, :sticky?, :blockdev?, :grpowned?,
        :setgid?, :executable_real?, :socket?, :owned?, :setuid?,
        :executable?

      attr_reader :fetch_thread
      attr_writer :size, :mtime
      attr_accessor :header_fetched
      private :size=, :mtime=, :header_fetched

      def initialize(url)
	unless parse_host(url)
	  raise Error::InvalidURI,
	    "Invalid HTTP URL '#{url}' was speficied."
	end
	super
	@mtime          = nil
	@size           = nil
	@fetch_thread   = nil
	@header_fetched = false
      end

      def size
	fetch_header if @size == nil
	wait_fetch   if @size == nil
	@size
      end

      def mtime
	fetch_header if @mtime == nil
	@mtime
      end
      alias :ctime :mtime
      alias :atime :mtime

      def ftype
	"file(http)"
      end

      def exists?
	begin
	  fetch_header
	rescue Net::ProtoFatalError # FIXME. original error class
	  return false
	end
	true
      end
      alias :exist? :exists?

      def wait_fetch
	if fetch_thread == nil
	  @fetch_thread = start_fetch
	end
	if fetch_thread.alive?
	  fetch_thread.join
	end
      end

      def fetch_header
	if header_fetched
	  log("header was already fetched. skip fetcing header.", Log::DEBUG)
	  return true
	end
	log("Start fetching header.")
	http_fetch(url, :head) { |rep|
	  if rep.key? "content-length"
	    log("Detected; Content-Length, set size; #{rep['content-length']}",
		Log::INFO)
	    @size = rep["content-length"].to_i
	  end
	  if rep.key? "last-modified"
	    log("Detected; Last-modified, set mtime; #{rep['last-modified']}",
		Log::INFO)
	    @mtime = Time.parse(rep["last-modified"])
	  end
	}
	log("Header was fetched")
	header_fetched = true
      end

      def start_fetch
	log("Start fetching thread.")
	Thread.start {
	  http_fetch(url, :get) { |rep|
	    buf << rep[0].body
	  }
	  buf.close
	  if @size == nil
	    @size = buf.size
	  end
	  log("HTTP fetching thread has been finished.")
	}
      end

      def open_as_read(*args, &block)
	wait_fetch
	super
      end

      def open_as_write(*args, &block)
	super # FXIME! only fail........
      end
      
    end # VFS::SchemeHandler::Http::File
end; end; end


if __FILE__ == $0
  f = "http://localhost/index.html"
#  f = "http://localhost/~sugi/"
  vfs = VFS::SchemeHandler::HTTP::File.new(f)
  p vfs
  p VFS::scheme(f)
  p vfs.scheme
  p vfs.mtime
#  p VFS::SchemeHandler::HTTP::File.mtime(f) # something wrong; warning: method redefined
  p vfs.size
#  p VFS::SchemeHandler::HTTP::File.size(f)
  p vfs.basename
  p vfs.dirname
  p vfs.scheme
  io = vfs.open
  puts io.read(20)
  io.close
  vfs.open { |io| puts io.read(20) }
#  io = VFS::SchemeHandler::HTTP::File.open(f)
#  puts io.read(20)
#  io.close
#  VFS::SchemeHandler::HTTP::File.open(f){ |io| puts io.read(20) }
end
