# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-04 12:45:54 +0900 (Sat, 04 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/common'

module VFS
  class Tempbuf
    StringMax    = 102400
    @@use_stringio = true

    def initialize(url)
      @io = nil
      @path = url
      begin
	require 'stringio'
	log("creating as StringIO.", Log::DEBUG)
	@io = StringIO.new
      rescue LoadError
	@@use_stringio = false
	switch_to_tempfile
      end
    end
    attr_reader :io, :path

    def switch_to_tempfile
      require 'tempfile'
      log("Switching to Tempfile.", Log::INFO)
      new_io = Tempfile.new('ruby-vfs')
      new_io << io.string if io && io.size > 0
      @io = new_io
    end

    def <<(str)
      io << str
      if @@use_stringio && StringIO === io && StringMax < io.size
	log("Buffer size was exceeded in StringIO", Log::INFO)
	switch_to_tempfile
      end
      io
    end

    def real_path
      io.path
    end

    def close
      log("fixme? close() was called. Hmmmmmmmm...", Log::DEBUG)
      if @@use_stringio && StringIO === io
	io.rewind
      elsif Tempfile === io
	io.flush
	io.rewind
      else
	raise Error::Unknown, "unkown error! please bug report!!"
      end
    end

    def method_missing(meth, *args, &block)
      if io.respond_to? meth
	io.send meth, *args, &block
      else
	raise NameError, "undefined method `#{meth}' for #{self.class}"
      end
    end

    private
    def log(*args)
      args[0] = "VFS(TempBuf): #{args[0]}"
      Log.log(*args)
    end
  end # Tempbuf

end # VFS
